//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      Sample/SoftParticle/FormFactorFuzzySphere.cpp
//! @brief     Implements class FormFactorFuzzySphere.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "Sample/SoftParticle/FormFactorFuzzySphere.h"
#include "Sample/LibFF/SomeFormFactors.h"
#include "Sample/Shapes/TruncatedEllipsoid.h"

FormFactorFuzzySphere::FormFactorFuzzySphere(const std::vector<double> P)
    : IFormFactor(P)
    , m_radius(m_P[0])
    , m_sigma(m_P[1])
{
    checkNodeArgs();
    m_shape3D = std::make_unique<TruncatedEllipsoid>(m_radius, m_radius, m_radius, 2 * m_radius, 0);
}

FormFactorFuzzySphere::FormFactorFuzzySphere(double mean, double sigma)
    : FormFactorFuzzySphere(std::vector<double>{mean, sigma})
{
}

complex_t FormFactorFuzzySphere::formfactor_at_bottom(C3 q) const
{
    double q2 = std::norm(q.x()) + std::norm(q.y()) + std::norm(q.z());
    double dw = std::exp(-q2 * m_sigma * m_sigma / 2.0);
    return dw * SampleUtils::someff::ffSphere(q, m_radius);
}
