//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      Sample/SoftParticle/FormFactorFuzzySphere.h
//! @brief     Defines and implements class FormFactorFuzzySphere.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_SAMPLE_SOFTPARTICLE_FORMFACTORFUZZYSPHERE_H
#define BORNAGAIN_SAMPLE_SOFTPARTICLE_FORMFACTORFUZZYSPHERE_H

#include "Sample/Particle/IFormFactor.h"
#include <memory>

//! A sphere with gaussian radius distribution.
//! @ingroup softParticle

class FormFactorFuzzySphere : public IFormFactor {
public:
    FormFactorFuzzySphere(std::vector<double> P);
    FormFactorFuzzySphere(double mean, double sigma);

    FormFactorFuzzySphere* clone() const override
    {
        return new FormFactorFuzzySphere(m_radius, m_sigma);
    }
    std::string className() const final { return "FormFactorFuzzySphere"; }
    // const auto tooltip = "class_tooltip";
    std::vector<ParaMeta> parDefs() const final
    {
        return {{"Radius", "nm", "para_tooltip", 0, +INF, 0},
                {"Fuzziness", "nm", "para_tooltip", 0, +INF, 0}};
    }

    double radialExtension() const override { return m_radius; }

    complex_t formfactor_at_bottom(C3 q) const override;

private:
    const double& m_radius;
    const double& m_sigma;
};

#endif // BORNAGAIN_SAMPLE_SOFTPARTICLE_FORMFACTORFUZZYSPHERE_H
